package api

import (
	"bytes"
	"encoding/base64"
	"fmt"
	"github.com/gin-gonic/gin"
	"go_backend/cmd/config"
	"go_backend/internal/dbs"
	"go_backend/internal/model"
	"gorm.io/gorm"
	"image"
	"image/png"
	"net/url"
	"os"
	"path/filepath"
	"strconv"
	"time"
)

type page struct {
	Limit  int `form:"limit" json:"limit"`
	Offset int `form:"offset" json:"offset"`
}

type detectionReturnParam struct {
	TotalNum        int64         `json:"totalNum"`
	DetectionRecord []DetectionVO `json:"detectionRecord"`
}

func GetDetectionRecord(router *gin.RouterGroup) {
	router.GET("/detection/List", func(c *gin.Context) {
		limit, _ := strconv.Atoi(c.Query("limit"))
		offset, _ := strconv.Atoi(c.Query("offset"))
		// 计算跳过的记录数
		skipRecord := (offset - 1) * limit
		detectionRecords := make([]model.PersonDetectionRecord, 0)
		db := dbs.GetGormDB()
		db.Limit(limit).Offset(skipRecord).Find(&detectionRecords)
		detectionVOS := make([]DetectionVO, 0)
		// 根据每条记录搜索对应的摄像头
		for _, record := range detectionRecords {
			var cameras []model.Camera
			result := db.Model(model.Camera{}).
				Where("camera_id = ?", record.CameraID).
				Find(&cameras)
			if result.Error != nil {
				fmt.Println(result.Error)
				return
			}
			if len(cameras) > 1 {
				fmt.Println("对应摄像头id ", record.CameraID, " 超过一个")
			}
			address := mergeAddress(cameras[0], db)
			detectionVOS = append(detectionVOS, transToDetectionVO(record, address))
		}

		var count int64
		db.Model(model.PersonDetectionRecord{}).Count(&count)
		Success(c, detectionReturnParam{
			TotalNum:        count,
			DetectionRecord: detectionVOS,
		})
	})
}

func mergeAddress(cameraTarget model.Camera, db *gorm.DB) string {
	var addressArray []string
	var address string
	var temp = cameraTarget
	for {
		addressArray = append(addressArray, temp.DisplayName)
		if temp.ParentCategoryID == 0 {
			break
		}
		var temp1 model.Camera
		db.Model(model.Camera{}).
			Where("id = ? ", temp.ParentCategoryID).
			Find(&temp1)
		temp = temp1
	}
	// 组织成xxx-xxx的形式
	for i := len(addressArray) - 1; i >= 1; i-- {
		address += addressArray[i]
		address += "-"
	}
	return address + addressArray[0]
}

type DetectionVO struct {
	ID                    int64  `json:"id"`
	PersonId              int32  `json:"person_id"`
	AlarmType             string `json:"alarm_type"`
	PersonType            string `json:"person_type"`
	DetectTime            string `json:"detect_time"`
	IpAddress             string `json:"ip_address"`
	Address               string `json:"address"`
	ProcessStatus         string `json:"process_status"`
	TrajectoryDetectionID int64  `json:"trajectory_detection_id"`
}

func transToDetectionVO(record model.PersonDetectionRecord, address string) DetectionVO {
	parsedUrl, err := url.Parse(record.Address)
	if err != nil {
		fmt.Println("Error parsing URL:", err)
	}
	ip := parsedUrl.Path
	return DetectionVO{
		ID:                    record.ID,
		PersonId:              record.PersonID,
		AlarmType:             record.AlarmType,
		PersonType:            record.PersonType,
		DetectTime:            record.DetectTime.Format("2006-01-02 15:04:05"),
		IpAddress:             ip,
		Address:               address,
		ProcessStatus:         record.ProcessStatus,
		TrajectoryDetectionID: record.TrajectoryDetectionID,
	}
}

type recordInfo struct {
	PersonType string `form:"personType" json:"personType"`
	PersonId   int    `form:"personId" json:"personId"`
	TimeStamp  int64  `form:"timestamp" json:"timestamp"`
	Picture    string `form:"picture" json:"picture"`
	CameraIP   string `form:"cameraIP" json:"cameraIP"`
}

func CreateDetectionRecord(router *gin.RouterGroup) {
	router.GET("/detection/create", func(c *gin.Context) {
		var req recordInfo
		err := c.ShouldBindJSON(&req)
		if err != nil {
			fmt.Println("ctx.ShouldBindJSON err: ", err)
			c.JSON(400, gin.H{"error": err.Error()})
			return
		}

		// 用base64解码图片数据
		decodedData, err := base64.StdEncoding.DecodeString(req.Picture)

		timestamp := req.TimeStamp
		tm := time.Unix(timestamp, 0)
		dir := filepath.Join(config.AllConfig.PicturePath, tm.Format("2006"),
			tm.Format("01"), tm.Format("02"),
			tm.Format("03"), tm.Format("04"))

		// 创建文件来保存图片，图片名字为personType-cameraip-time-personid
		fileName := fmt.Sprintf("%s.%s.%s.%s.png", req.PersonType, req.CameraIP, tm.Second(), tm.Format("20240102150405"))
		// 创建目录
		err = os.MkdirAll(dir, os.ModePerm)
		if err != nil {
			fmt.Println("创建目录出错:", err)
			return
		}

		filePath := filepath.Join(dir, fileName)
		// 创建文件
		file, err := os.Create(filePath)
		if err != nil {
			fmt.Println("创建文件出错:", err)
			return
		}
		defer file.Close()
		// 使用PNG格式保存图片
		img, _, err := image.Decode(bytes.NewReader(decodedData))
		if err != nil {
			fmt.Println("读取图片出错:", err)
			return
		}
		err = png.Encode(file, img)
		if err != nil {
			fmt.Println("保存图片出错:", err)
			return
		}

		var cameras []model.Camera
		dbs.GetGormDB().Model(model.PersonDetectionRecord{}).
			Where("ip_address like ?", "%"+req.CameraIP+"%").
			Find(&cameras)

		if len(cameras) != 1 {
			fmt.Println("该ip对应的摄像头大于1个:", req.CameraIP)
			return
		}

		newRecord := model.PersonDetectionRecord{
			PersonType: req.PersonType,
			Address:    req.CameraIP,
			DetectTime: tm,
			CameraID:   string(cameras[0].ID),
		}
		dbs.GetGormDB().Model(model.PersonDetectionRecord{}).Create(&newRecord)

		Success(c, "success")
	})
}
